parser

package
v0.3.0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Sep 26, 2025 License: GPL-3.0 Imports: 2 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func Value

func Value(raw string, escapeComma bool) ([]string, error)

Value parses a tag value. The value is split on comma, and escaped commas are ignored.

Types

type Filler

type Filler[T any] interface {
	Data() T
	Fill(key, value string) error
}

Filler is the interface implemented by types that can fill elements from a struct tag.

Jump to