Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Entry ¶
Entry represents a struct tag entry. An entry is composed of a key and a value.
type Filler ¶
type Filler struct {
// contains filtered or unexported fields
}
Filler fills the tag from a struct tag.
type Options ¶
type Options struct {
// EscapeComma is used to escape the comma character within the value.
EscapeComma bool
// AllowDuplicateKeys allows duplicate keys.
// NOT RECOMMENDED TO USE IT: this does not follow the struct tag conventions.
AllowDuplicateKeys bool
}
Options for the parser.
type Tag ¶
type Tag struct {
// contains filtered or unexported fields
}
Tag represents a struct tag.
Click to show internal directories.
Click to hide internal directories.